@@ -21,12 +21,6 @@ class TionDelegation(DefaultDelegate):
         _LOGGER.debug("Got data in %d response %s", handle, bytes(data).hex())
         self.__topic.read()
 
-    def handleDiscovery(self, dev, isNewDev, isNewData):
-        if isNewDev:
-            _LOGGER.debug("Discovered device %s", dev.addr)
-        elif isNewData:
-            _LOGGER.debug("Received new data from %s", dev.addr)
-
     def setReadTopic(self, topic):
         self.__topic = topic
 
@@ -351,17 +345,16 @@ class tion(TionDummy):
 
     def _enable_notifications(self):
         _LOGGER.debug("Enabling notification")
-        setup_data = b"\x01\x00"
-
-        _LOGGER.debug("Notify handler is %s", self.notify.getHandle())
-        notify_handle = self.notify.getHandle() + 1
-
-        _LOGGER.debug("Will write %s to %s handle", setup_data, notify_handle)
-        result = self._btle.writeCharacteristic(notify_handle, setup_data, withResponse=True)
-        _LOGGER.debug("Result is %s", result)
         self._btle.withDelegate(self._delegation)
-        self.notify.read()
+        _LOGGER.debug("Delegation enabled")
+        try:
+            self.notify.read()
+            _LOGGER.debug("First read done")
+        except Exception as e:
+            _LOGGER.warning("Got exception '%s' while first read, but it is OK and we may ignore it", str(e))
+
         self._delegation.setReadTopic(self.notify)
+        _LOGGER.debug("enable_notification is done")
 
     @property
     def fan_speed(self):
@@ -470,3 +463,28 @@ class tion(TionDummy):
         :return: integer equivalent of mode
         """
         return self.modes.index(mode) if mode in self.modes else 2
+
+    def pair(self):
+        _LOGGER.debug("Pairing")
+        self._connect()
+        _LOGGER.debug("Connected. BT pairing ...")
+        try:
+            # use private methods to avoid disconnect if already paired
+            self._btle._writeCmd('pair' + '\n')
+            rsp = self._btle._waitResp('mgmt')
+            if rsp['estat'][0] == 0 or rsp['estat'][0] == 19 or rsp['code'][0] == 'success':
+                _LOGGER.debug(rsp['emsg'][0])
+            else:
+                _LOGGER.warning("Unexpected response: %s(%d)", rsp['emsg'][0], rsp['estat'][0])
+                raise TionException(rsp['estat'][0], rsp['emsg'][0])
+            # device-specific pairing
+            _LOGGER.debug("Device-specific pairing ...")
+            self._pair()
+            _LOGGER.debug("Device pair is done")
+        finally:
+            _LOGGER.debug("disconnected")
+            self._disconnect()
+
+    @abc.abstractmethod
+    def _pair(self):
+        """Perform model-specific pair steps"""
